#!/bin/bash

set -x;
set -e
set -o pipefail

source "/src/emsdk/emsdk_env.sh"

mkdir /build

OUT="webhibiki"

if [[ "$2" != "" ]]; then
  OUT="$2"
fi

export LIBRARY_PATH="${LIBRARY_PATH}:/build"
export LD_LIBRARY_PATH="${LIBRARY_PATH}:/build"



export CFLAGS="${CFLAGS} -I/build/include ${1} -flto -O3 -ffast-math -g0 -s EVAL_CTORS=1 -s ENVIRONMENT=web,worker -s WASM_BIGINT=1 -s MALLOC=emmalloc -s AUTO_NATIVE_LIBRARIES=0"
export CXXFLAGS="${CXXFLAGS} -I/build/include ${1} -flto -O3 -ffast-math -g0 -s EVAL_CTORS=2 -s ENVIRONMENT=web,worker -s WASM_BIGINT=1 -s MALLOC=emmalloc -s AUTO_NATIVE_LIBRARIES=0"
export LDFLAGS="${CXXFLAGS} -g0 -flto -s EVAL_CTORS=2 -s ENVIRONMENT=web,worker -s WASM_BIGINT=1 -s MALLOC=emmalloc -s AUTO_NATIVE_LIBRARIES=0"
export PATH="${PATH}:/build"

git clone https://git.gammaspectra.live/S.O.N.G/c-gaborator.git /src/c-gaborator
pushd /src/c-gaborator
git reset --hard 8ef1b52d3b3ec8c06cfc7b5592b714aedf73e708
git submodule update --init --recursive

mkdir build && pushd build
sed -i 's/USE_TYPE_DOUBLE "" ON/USE_TYPE_DOUBLE "" OFF/' ../CMakeLists.txt
if [[ "$CFLAGS" == *"msimd128"* ]]; then
  #disable alignment checks as instructions will be handled by WebAssembly
  sed -i 's/define VALIGNED(ptr) ((((uintptr_t)(ptr)) & 0xF) == 0)/define VALIGNED(ptr) 1/' ../lib/pffft/simd/pf_sse1_float.h
  emcmake cmake .. -DCMAKE_BUILD_TYPE=Release \
  -DCMAKE_CXX_FLAGS_RELEASE="-march=native -D_M_IX86" -DCMAKE_C_FLAGS_RELEASE="-march=native -D_M_IX86" \
  -DCMAKE_INSTALL_PREFIX="/build"
else
  emcmake cmake .. -DCMAKE_BUILD_TYPE=Release \
  -DCMAKE_CXX_FLAGS_RELEASE="-march=native -DPFFFT_SIMD_DISABLE" -DCMAKE_C_FLAGS_RELEASE="-march=native -DPFFFT_SIMD_DISABLE" \
  -DCMAKE_INSTALL_PREFIX="/build"
fi
emmake make install
cp -v lib/pffft/libpffft.a /build/lib

git clone https://github.com/libsndfile/libsamplerate.git /src/libsamplerate
pushd /src/libsamplerate
git reset --hard aee38d0bc797d0d1a3774ef574af1d5d248d2398
mkdir build && pushd build
emcmake cmake .. -DCMAKE_BUILD_TYPE=Release \
-DLIBSAMPLERATE_EXAMPLES=OFF -DBUILD_TESTING=OFF \
-DLIBSAMPLERATE_ENABLE_SINC_BEST_CONVERTER=OFF -DLIBSAMPLERATE_ENABLE_SINC_MEDIUM_CONVERTER=OFF -DLIBSAMPLERATE_ENABLE_SINC_FAST_CONVERTER=OFF \
-DLIBSAMPLERATE_INSTALL_PKGCONFIG_MODULE=OFF \
-DCMAKE_CXX_FLAGS_RELEASE="-march=native -O3 -ffast-math " -DCMAKE_C_FLAGS_RELEASE="-march=native -O3 -ffast-math " \
-DCMAKE_INSTALL_PREFIX="/build"
emmake make install


mkdir /build/cpp && pushd /build/cpp
emcmake cmake /src/dist/cpp -DCMAKE_BUILD_TYPE=Release \
-DCMAKE_CXX_FLAGS_RELEASE="-march=native" -DCMAKE_C_FLAGS_RELEASE="-march=native" \
-DCMAKE_INSTALL_PREFIX="/build"

emmake make install

pushd /build

ls -lah /build/lib

emcc /build/lib/libhibiki.a /build/lib/libcgaborator.a /build/lib/libpffft.a /build/lib/libsamplerate.a \
--pre-js /src/dist/js/pre.js --post-js /src/dist/js/post.js \
${1} -O3 -ffast-math -g0 -s ENVIRONMENT=web,worker -s WASM_BIGINT=1 \
-s ENVIRONMENT=web,worker -s EVAL_CTORS=2 \
-s ALLOW_MEMORY_GROWTH=1 -s INITIAL_MEMORY=32mb -s MEMORY64=0 -s MALLOC=emmalloc -s NO_EXIT_RUNTIME=1 -s AUTO_NATIVE_LIBRARIES=0 \
-s EXPORTED_FUNCTIONS="_malloc,_free,_new_point_processor,_point_processor_process,_point_processor_finish,_delete_point_processor" \
-s EXPORTED_RUNTIME_METHODS="['ccall', 'cwrap', 'fromArrayBuffer']" \
-s LEGACY_VM_SUPPORT=0 -s MIN_CHROME_VERSION=91 -s MIN_FIREFOX_VERSION=89 -s MIN_EDGE_VERSION=0x7FFFFFFF -s MIN_SAFARI_VERSION=152000 \
-flto --llvm-lto 1 --memory-init-file=0 \
--closure 1 \
-s MODULARIZE=1 -s EXPORT_ES6=1 -s USE_ES6_IMPORT_META=0 \
-s WASM=1 -o "${OUT}".js

ls -lah ./

cp -v "${OUT}".* /output