
const PANAKO_AUDIO_BLOCK_SIZE = 1 << 17;
const PANAKO_AUDIO_DEFAULT_RESOLUTION = 8;
const PANAKO_SAMPLE_RATE = 16000;

function processSlice (pointProcessor, slice, buf) {
    Module["HEAPU8"].set(new Uint8Array(slice.buffer, slice.byteOffset, slice.length*slice.BYTES_PER_ELEMENT), buf);
    Module["ccall"]('point_processor_process', null, ['number', 'number', 'number'], [pointProcessor, buf, slice.length]);
}


Module = Module || {};

Module["fromArrayBuffer"] = async (buffer, options) => {
    options = options || {};
    let compactHash = options["compactHash"] || true;
    let packedPrint = options["packedPrint"] || true;
    let timeResolutionInMs = options["timeResolutionInMs"] || PANAKO_AUDIO_DEFAULT_RESOLUTION;

    let audioCtx = new AudioContext({
        latencyHint: "balanced",
        //sampleRate: PANAKO_SAMPLE_RATE //TODO: maybe this can be changed once there is a proper api
    });

    let audioData = await audioCtx.decodeAudioData(buffer);
    if(options["progressCallback"]){
        options["progressCallback"](0);
    }

    let pointProcessor = Module["ccall"]('new_point_processor', 'number', ['number', 'number', 'number'], [audioData.sampleRate, audioData.numberOfChannels, timeResolutionInMs]);

    let channels = [];
    for (let i = 0; i < audioData.numberOfChannels; ++i){
        channels.push(audioData.getChannelData(i));
    }

    let interleaved = new Float32Array(PANAKO_AUDIO_BLOCK_SIZE*audioData.numberOfChannels);
    let buf = Module["_malloc"](PANAKO_AUDIO_BLOCK_SIZE*audioData.numberOfChannels * interleaved.BYTES_PER_ELEMENT);

    let currentLength = 0;
    let totalSamples = 0;
    let blockNumber = 0;
    for(let i = 0; i < audioData.length; ++i){
        //Interleave
        for (let j = 0; j < audioData.numberOfChannels; ++j){
            interleaved[currentLength++] = channels[j][i];
        }

        if (currentLength === (PANAKO_AUDIO_BLOCK_SIZE*audioData.numberOfChannels)){
            console.log("hibiki: processing block " + blockNumber);
            //Do this to yield to render/main thread
            await (new Promise((resolve => {
                setTimeout(() => {
                    processSlice(pointProcessor, interleaved, buf);
                    resolve();
                }, 0);
            })));

            totalSamples += currentLength;
            currentLength = 0;
            ++blockNumber;

            if(options["progressCallback"]){
                options["progressCallback"](totalSamples / (audioData.length*audioData.numberOfChannels));
            }
        }
    }

    if (currentLength > 0){
        console.log("finish block " + blockNumber + " length " + currentLength);
        //Send remaining data
        totalSamples += currentLength;
        processSlice(pointProcessor, interleaved.subarray(0, currentLength), buf);
    }


    if(options["progressCallback"]){
        options["progressCallback"](totalSamples / (audioData.length*audioData.numberOfChannels));
    }

    console.log("hibiki: js total samples " + totalSamples);

    let intPtr = Module["_malloc"](4);
    let resultPtr = Module["ccall"]('point_processor_finish', 'number', ['number', 'number', 'number'], [pointProcessor, compactHash, packedPrint, intPtr]);
    let resultLength = (new Int32Array(Module["HEAPU8"].buffer, intPtr, 1))[0];
    Module["_free"](intPtr);
    Module["_free"](buf);
    //Copy result
    let printResult = new Uint8Array(new Uint8Array(Module["HEAPU8"].buffer, resultPtr, resultLength));

    Module["ccall"]('delete_point_processor', null, ['number'], [pointProcessor]);

    return printResult;
}